package com.hzya.frame.u8.base.inventory.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.hzya.frame.basedao.service.impl.BaseService;
import com.hzya.frame.serviceUtil.ServiceUtil;
import com.hzya.frame.u8.base.identity.entity.UAIdentityEntity;
import com.hzya.frame.u8.base.identity.service.IUAIdentityService;
import com.hzya.frame.u8.base.inventory.dao.IInventoryDao;
import com.hzya.frame.u8.base.inventory.entity.BasPartEntity;
import com.hzya.frame.u8.base.inventory.entity.InventoryEntity;
import com.hzya.frame.u8.base.inventory.entity.InventorySubEntity;
import com.hzya.frame.u8.base.inventory.service.IBasPartService;
import com.hzya.frame.u8.base.inventory.service.IInventoryExtService;
import com.hzya.frame.u8.base.inventory.service.IInventoryService;
import com.hzya.frame.u8.base.inventory.service.IInventorySubService;
import com.hzya.frame.web.exception.BaseSystemException;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Description
 * @Author xiangerlin
 * @Date 2025/3/13 15:39
 **/
@Service
public class InventoryService extends BaseService<InventoryEntity,String> implements IInventoryService {

    Logger logger = LogManager.getLogger(getClass());

    private IInventoryDao inventoryDao;

    @Autowired
    public void setInventoryDao(IInventoryDao dao) {
        this.inventoryDao = dao;
        this.dao = dao;
    }
    @Autowired
    private IInventorySubService subService;
    @Autowired
    private IBasPartService basPartService;
    @Autowired
    private IUAIdentityService uaIdentityService;
    @Resource
    private IInventoryExtService inventoryExtService;
    /**
     * 查询存货档案
     *
     * @param entity
     * @return
     * @throws Exception
     */
    @DS("#entity.dataSourceCode")
    @Override
    public List<InventoryEntity> queryList(InventoryEntity entity) throws Exception {
        List<InventoryEntity> List = inventoryDao.query(entity);
        return List;
    }

    /**
     * 新增存货档案
     *
     * @param entity
     * @throws Exception
     */
    @DS("#entity.dataSourceCode")
    @Override
    public void saveInventory(InventoryEntity entity) throws Exception {
        checkInventory("save",entity);
        //保存存货信息
        inventoryDao.save(entity);
        //保存存货副表
        subService.saveOrUpdateInvSub(entity.getInvSubEntity());
        //保存扩展自由项
        inventoryExtService.saveOrUpdateInv(entity.getInvExtEntity());
        //保存自定义表
        //先查一下bas_part有没有
        BasPartEntity part = new BasPartEntity();
        part.setInvCode(entity.getcInvCode());
        part.setDataSourceCode(entity.getDataSourceCode());
        part.setAcc_name(entity.getAcc_name());
        List<BasPartEntity> partList = basPartService.query(part);
        if (CollectionUtils.isEmpty(partList)){
            UAIdentityEntity uaIdentityEntity = new UAIdentityEntity();
            uaIdentityEntity.setCacc_id(entity.getCacc_id());
            uaIdentityEntity.setcVouchType("bas_part");
            uaIdentityEntity.setDataSourceCode(entity.getDataSourceCode());
            uaIdentityEntity.setAcc_name(entity.getAcc_name());
            List<UAIdentityEntity> identityList =  uaIdentityService.query(uaIdentityEntity);
            if (CollectionUtils.isNotEmpty(identityList)){
                UAIdentityEntity uaIdentity = identityList.get(0);
                if (null != uaIdentity){
                    BasPartEntity basPartEntity = new BasPartEntity();
                    basPartEntity.setInvCode(entity.getcInvCode());
                    Integer id_num_add = uaIdentity.getiFatherId()+1;// +1
                    long max_id = 1000000000 + id_num_add;//前面拼接上1000000000
                    //更新最大编号
                    uaIdentity.setCacc_id(entity.getCacc_id());
                    uaIdentity.setcVouchType("bas_part");
                    //最大号+1
                    UAIdentityEntity identity = new UAIdentityEntity();
                    identity.setCacc_id(uaIdentity.getCacc_id());
                    identity.setcVouchType(uaIdentity.getcVouchType());
                    identity.setiFatherId(uaIdentity.getiFatherId());
                    identity.setiChildId(uaIdentity.getiChildId());
                    identity.setDataSourceCode(entity.getDataSourceCode());
                    identity.setAcc_name(entity.getAcc_name());
                    uaIdentityService.updateMaxSaleOrderId(identity);
                    basPartEntity.setPartId(String.valueOf(max_id));
                    basPartEntity.setInvCode(entity.getcInvCode());
                    basPartEntity.setDataSourceCode(entity.getDataSourceCode());
                    logger.info("======物料编码"+entity.getcInvCode()+"，生成的bas_part最大id"+max_id);
                    basPartEntity.setAcc_name(entity.getAcc_name());
                    basPartService.save(basPartEntity);
                }
            }
        }
    }

    /**
     * 修改存货档案
     *
     * @param entity
     * @throws Exception
     */
    @DS("#entity.dataSourceCode")
    @Override
    public void updateInventory(InventoryEntity entity) throws Exception {
       checkInventory("update",entity);
        inventoryDao.update(entity);
        //更新存货自由项
        inventoryExtService.saveOrUpdateInv(entity.getInvExtEntity());
        //更新存货子表
        subService.saveOrUpdateInvSub(entity.getInvSubEntity());
    }

    /**
     * 必填校验
     * @param type
     * @param entity
     */
    private void checkInventory(String type,InventoryEntity entity){
        Assert.notNull(entity,()-> new BaseSystemException("参数不能为空"));
        Assert.notEmpty(entity.getcInvCode(),()-> new BaseSystemException("存货编码不能为空"));
        Assert.notEmpty(entity.getDataSourceCode(),()-> new BaseSystemException("数据源编码不能为空"));
        Assert.notEmpty(entity.getAcc_name(),()-> new BaseSystemException("账套号不能为空"));
        Assert.notNull(entity.getInvExtEntity(),()-> new BaseSystemException("扩展自定义项不能为空"));
        Assert.notNull(entity.getInvSubEntity(),()-> new BaseSystemException("存货子表不能为空"));
        switch (type) {
            case "save":
                break;
            case "update":
                break;
        }
    }
}
